"""Test whether the CI can access some basic API endpoints"""

from unittest import TestCase

from veikkaaja.veikkaus_client import GameTypes

from .mock_client import MockClient


class TestEbetBetting(TestCase):
    """test access to API"""

    def test_parsing_queried_ebet_events(self):
        """Set up an invalid gametype"""
        client = MockClient()
        games = client.upcoming_events(GameTypes.EBET)

        self.assertIsNotNone(games)
        self.assertEqual(len(games), 269)

        found_roda_telstar_game = False
        for game in games:
            self.assertIsNotNone(game)

            # info for a single specific game
            if game.home_team == "Roda JC" and game.away_team == "Telstar":
                found_roda_telstar_game = True
                self.assertEqual(game.draw_type, "1X2")
                self.assertEqual(game.event_id, "98744678")
                self.assertEqual(game.row_id, "2186938")
                self.assertEqual(game.status, "OPEN")
                self.assertEqual(game.home_odds, 188.0)
                self.assertEqual(game.draw_odds, 375.0)
                self.assertEqual(game.away_odds, 320.0)

        self.assertTrue(found_roda_telstar_game)



"""Main veikkaus client module"""
import json
import os
from datetime import datetime
from enum import Enum
from typing import Any, Dict, List, NamedTuple, Union

import requests

from veikkaaja import logger
from veikkaaja.endpoints import EndPoint


class GameTypes(Enum):
    """Available gamemodes in the API"""
    MULTISCORE = "MULTISCORE"  # Moniveto
    SCORE = "SCORE"  # Tulosveto
    SPORT = "SPORT"  # Vakio
    WINNER = "WINNER"  # Voittajavedot
    PICKTWO = "PICKTWO"  # Päivän pari
    PICKTHREE = "PICKTHREE"  # Päivän trio
    PERFECTA = "PERFECTA"  # Superkaksari
    TRIFECTA = "TRIFECTA"  # Supertripla
    EBET = "EBET"  # Pitkäveto
    RAVI = "RAVI"  # Moniveikkaus


class BetTarget(Enum):
    """Currently only 1x2 supported"""
    HOME = "HOME"
    X = "X"
    AWAY = "AWAY"


class BetDecision(NamedTuple):
    """Currently only 1x2 supported"""
    # what to be
    target: BetTarget
    # how much to bet in cents
    amount: int


class Game:
    """A class for holding EBET event information"""

    # pylint:disable=too-many-instance-attributes
    # This is intended just as a wrapper to hold the
    # data in the API response

    home_team = ""
    away_team = ""
    home_odds = 0.0
    away_odds = 0.0
    draw_odds = 0.0
    event_id = 0
    row_id = 0
    draw_type = ""  # 1x2
    status = ""
    brand_name = 0
    close_time = datetime.fromtimestamp(0)
    league = ""
    sport_id = 0

    def __init__(self, client: 'VeikkausClient'):
        """"""
        self._client: VeikkausClient = client

    def place_bet(self, bet: BetDecision):
        """Given amount in cents, bet for target."""
        self._client.place_bet(self, bet)

    def __repr__(self):
        """Make nicer output"""
        close_str = self.close_time.strftime("%d.%m.%Y %H:%M")
        return f"{self.__class__.__name__:} type: '{self.draw_type:3}' {close_str} {self.league}: {self.home_team:15} - {self.away_team:15} id: {self.row_id} event_id: {self.event_id} status: {self.status}, odds: ({self.home_odds:6} - {self.draw_odds:6} - {self.away_odds:6})"  #pylint:disable=line-too-long


class EventInfo:
    """A wrapper to keep information of the EBET draws"""
    league = ""
    external_id = ""

    def __repr__(self):
        return f"{self.__class__.__name__}: league: {self.league}, external_id: {self.external_id}"


class VeikkausClient:
    """The main client that holds on the API session"""

    API_HEADERS = {
        "Content-Type": "application/json",
        "Accept": "application/json",
        "X-ESA-API-Key": "ROBOT"
    }

    def __init__(self, account="", password=""):
        """
        Arguments:
            account (str):  Name of the account or empty if empty
                            account name is loaded from VEIKKAUS_ACCOUNT
                            environment variable.
            password (str): account password. If empty, loaded from
                            VEIKKAUS_PASSWORD environment variable
        """

        acc_password = password
        if not acc_password:
            if "VEIKKAUS_PASSWORD" not in os.environ:
                raise RuntimeError("Missing account authentication information")
            acc_password = os.environ['VEIKKAUS_PASSWORD']

        acc = account
        if not acc:
            if "VEIKKAUS_ACCOUNT" not in os.environ:
                raise RuntimeError("Missing account authentication information")
            acc = os.environ['VEIKKAUS_ACCOUNT']

        self.session = self.login(acc, acc_password)

    def _access_endpoint(self,
                         endpoint: EndPoint,
                         payload: Dict[str, Any] = None,
                         method="GET") -> Union[requests.Response, None]:
        """
        A common wrapper for sending and logging API requests

        Arguments:
            endpoint: the url of the endpoint
            payload: dictionary of the query parameters
            method: GET or POST
        """
        payload = {} if payload is None else payload

        if not self.session:
            logger.warning("No active session for accessing '%s'.", endpoint.endpoint)
            return None

        # log sending out a request
        payload_text = "\n{}".format(json.dumps(payload, indent=4)) if payload else ""
        logger.info("\033[93mSending\033[0m %s %s:%s", method, endpoint.url, payload_text)

        self.save_outgoing_request(endpoint, payload)

        if method == "GET":
            response = self.session.get(
                endpoint.url, headers=self.API_HEADERS, params=payload)
        elif method == "POST":
            response = self.session.post(
                endpoint.url, headers=self.API_HEADERS, json=payload)
        else:
            raise RuntimeError("Unsupported method {}".format(method))

        self.save_incoming_response(endpoint, response)

        if response.status_code != 200:
            # log out the error
            logger.warning("hello")
            logger.error("\033[91mRequest failed\033[0m %s, %s. URL: %s",
                         response.status_code, response.reason, response.url)

            # RED debug log entry
            if response.content:
                logger.debug("\033[91mInvalid request:\033[0m\n%s", response.content)
            return None

        # green dedub log entry, the responses are quite large
        logger.info("\033[92mResponse OK\033[0m from %s", endpoint.endpoint)
        logger.debug("\033[92mReceived:\033[0m\n%s",
                     json.dumps(response.json(), indent=4))

        return response

    def save_outgoing_request(self, endpoint: EndPoint, payload: Dict[Any, Any]):
        """For testing, add and interface for saving the outgoing messages."""

    def save_incoming_response(self, endpoint: EndPoint, response: requests.Response):
        """For testing, add and interface for saving the incoming responses."""

    def login(self, account: str, password: str):
        """
        Starts and returns a requests session.

        Returns:
            requests.Session or None if login failed.
        """
        login_payload = {"type": "STANDARD_LOGIN", "login": account, "password": password}
        logger.info("Trying to log in...")
        logger.info("\033[93mSending\033[0m %s %s", "GET",
                    EndPoint.login_endpoint().endpoint)
        session = requests.Session()
        response = session.post(
            EndPoint.login_endpoint(),
            data=json.dumps(login_payload),
            headers=self.API_HEADERS)

        if response.status_code != 200:
            logger.error("Cannot login")
            return None

        logger.info("\033[92mResponse OK\033[0m Succesfully logged in!")
        return session

    def get_balance(self, balance="usableBalance"):
        """Return the account balance
        Args:
            balance (str): the type of balance to return, options are
                           ('balance', 'usableBalance', 'frozenBalance')
        """
        assert balance in ('balance', 'usableBalance',
                           'frozenBalance'), "Invalid balance type"

        response = self._access_endpoint(EndPoint.account_info_endpoint(), method="GET")

        if response is None:
            return 0

        cash = response.json().get('balances', {}).get('CASH', {})
        logger.info("Account has balance: total: %s €, frozen: %s €, usable: %s €",
                    cash.get('balance', 0) / 100,
                    cash.get('frozenBalance', 0) / 100,
                    cash.get('usableBalance', 0) / 100)

        # return the requested balance
        return cash.get(balance, 0) / 100

    def upcoming_events(self, game_type: GameTypes) -> List[Game]:
        """Get upcoming games"""

        payload = {'game-names': game_type.value}
        response = self._access_endpoint(
            EndPoint.games_info_endpoint(), payload=payload, method="GET")

        if not response:
            return []

        data = response.json()

        if game_type == GameTypes.EBET:
            return self.parse_draws(data)

        logger.warning("Not yet implemented game type: %s", game_type.value)
        return []

    def parse_draws(self, data: Dict):
        """
        API response:

            "draws": [
            {
                "gameName": "EBET",
                "brandName": "838",
                "id": "2143963",
                "name": "SINGLE",
                "status": "OPEN",
                "openTime": 1600398000000,
                "closeTime": 1600887480000,
                "drawTime": 1600887600000,
                "resultsAvailableTime": 1600894799000,
                "gameRuleSet": {
                    "basePrice": 100,
                    "maxPrice": 1000000,
                    "stakeInterval": 10,
                    "minStake": 10,
                    "maxStake": 100000,
                    "minSystemLevel": 1,
                    "maxSystemLevel": 10,
                    "oddsType": "FIXED"
                },
                "rows": [
                    {
                        "id": "1",
                        "status": "OPEN",
                        "includedRowCount": 32,
                        "name": "",
                        "description": "",
                        "detailedDescription": "1/2",
                        "tvChannel": "",
                        "competitors": [
                            {
                                "id": "1",
                                "name": "Olympiakos",
                                "number": 133,
                                "odds": {
                                    "odds": 132
                                },
                                "status": "ACTIVE",
                                "handicap": "0.00"
                            },
                            {
                                "id": "2",
                                "name": "Omonoia",
                                "number": 313,
                                "odds": {
                                    "odds": 860
                                },
                                "status": "ACTIVE"
                            },
                            {
                                "id": "3",
                                "name": "Tasapeli",
                                "odds": {
                                    "odds": 440
                                },
                                "status": "ACTIVE"
                            }
                        ],
                        "eventId": "98723990",
                        "excludedEvents": [
                            "98723990"
                        ],
                        "type": "1X2",
                        "sportId": "1",
                        "externalId": "0"
                    }
                ]
            },

        """

        games = []
        for entry in data.get('draws', []):

            good = True
            game = Game(self)
            game.row_id = entry.get('id')
            game.brand_name = entry.get('brandName')
            game.status = entry.get('status')
            game.close_time = datetime.fromtimestamp(entry.get('closeTime', 0) / 1000)
            for row in entry.get('rows', []):

                game.event_id = row.get('eventId')
                game.status = row.get('status')
                game.draw_type = row.get('type')
                game.sport_id = row.get('sportId')
                for comp in row.get('competitors', []):
                    if comp.get('id') == "1":
                        game.home_team = comp.get('name')
                        game.home_odds = float(comp.get('odds').get('odds'))
                    if comp.get('id') == "2":
                        game.away_team = comp.get('name')
                        game.away_odds = float(comp.get('odds').get('odds'))
                    if comp.get('id') == "3":
                        if not comp.get('name') == "Tasapeli":
                            logger.debug("Skipping %s, since it has no 'Tasapeli' odds",
                                         row.get('name'))
                            good = False
                        game.draw_odds = float(comp.get('odds').get('odds'))

            if good:
                games.append(game)

        games = sorted(games, key=lambda game: game.close_time)
        return games

    def sport_types(self) -> List[Dict[str, str]]:
        """query available sport type ids:

        API Response:

        [
            {
                "id": "7",
                "name": "Salibandy"
            },
            {
                "id": "48",
                "name": "Arvontapelit"
            },
            {
                "id": "25",
                "name": "Kamppailulajit"
            }
        ]
        """
        payload = {'lang': "fi"}
        response = self._access_endpoint(
            EndPoint.sport_type_code_endpoint(), payload, method="GET")

        if not response:
            return []

        return response.json()

    def sport_categories(self, sport_id: int) -> List[Dict[str, str]]:
        """
        query available sport type subgateries

        e.g. for football query different countries
        that have football leagues.

        API Response:

        {
            "id": "1",
            "name": "Jalkapallo",
            "categories": [
                {
                    "id": "1",
                    "name": "Suomi"
                },
                {
                    "id": "2",
                    "name": "Englanti"
                },
                {
                    "id": "3",
                    "name": "Italia"
                },
        }
        """
        payload = {'lang': "fi"}
        response = self._access_endpoint(
            EndPoint.sport_categories_endpoint(sport_id), payload, method="GET")

        if not response:
            return []

        return response.json()

    def sport_tournaments(self, sport_id: int,
                          sport_category_id: int) -> List[Dict[str, str]]:
        """
        query available tournaments for sport type subgateries

        e.g. for football query different countries
        that have football leagues.

        API Response:
        {
            "id": "2",
            "name": "Englanti",
            "tournaments": [
                {
                    "id": "1",
                    "name": "Valioliiga"
                },
                {
                    "id": "2",
                    "name": "Mestaruussarja"
                },
                {
                    "id": "3",
                    "name": "Ykk\u00f6sliiga"
                },
            ]
        }
        """

        payload = {'lang': "fi"}
        response = self._access_endpoint(
            EndPoint.sport_tournaments_endpoint(sport_id, sport_category_id),
            payload,
            method="GET")

        if not response:
            return []

        return response.json()

    def sport_tournament_info(self, sport_id: int, sport_category_id: int,
                              sport_tournament_id) -> List[Dict[str, str]]:
        """
        query available tournaments for sport type subgateries

        e.g. for football query different countries
        that have football leagues.

        API Response:
        {
            "id": "1",
            "name": "Valioliiga",
            "events": [
                {
                    "id": "94772195",
                    "name": "Crystal P - Bournemouth",
                    "date": 1557669600000
                },
                ...
            ],
            "teams": [
                {
                    "id": "60",
                    "name": "Huddersfield",
                    "shortName": "Huddersfield"
                },
                {
                    "id": "446",
                    "name": "Hull",
                    "shortName": "Hull"
                },
                ...
            ],
        }

        """

        payload = {'lang': "fi"}
        response = self._access_endpoint(
            EndPoint.sport_tournament_info_endpoint(sport_id, sport_category_id,
                                                    sport_tournament_id),
            payload,
            method="GET")

        if not response:
            return []

        return response.json()

    def event_info(self, event_id: int) -> Union[EventInfo, None]:
        """Query more specific information for the event

        API response:
            {
                "id": "98587029",
                "name": "Liverpool - Arsenal",
                "sportId": "1",
                "sportName": "Jalkapallo",
                "categoryId": "2",
                "categoryName": "Englanti",
                "tournamentId": "1",
                "tournamentName": "Valioliiga",
                "teams": [
                    {
                        "id": "1",
                        "name": "Arsenal",
                        "shortName": "Arsenal"
                    },
                    {
                        "id": "9",
                        "name": "Liverpool",
                        "shortName": "Liverpool"
                    }
                ],
                "date": 1601319600000,
                "externalId": "23203829",
                "hasLiveBetting": false
            }

        """

        payload = {'lang': "fi"}
        response = self._access_endpoint(
            EndPoint.single_event_info_endpoint(event_id), payload, method="GET")

        if not response:
            return None

        data = response.json()

        event = EventInfo()
        event.league = data.get('tournamentName')
        event.external_id = data.get('externalId')

        return event

    def draw_info(self, draw_id: int) -> Union[EventInfo, None]:
        """Query more specific information for a single draw

        API response:
        {
            "id": "98587029",
            "name": "Liverpool - Arsenal",
            "sportId": "1",
            "sportName": "Jalkapallo",
            "categoryId": "2",
            "categoryName": "Englanti",
            "tournamentId": "1",
            "tournamentName": "Valioliiga",
            "teams": [
                {
                    "id": "1",
                    "name": "Arsenal",
                    "shortName": "Arsenal"
                },
                {
                    "id": "9",
                    "name": "Liverpool",
                    "shortName": "Liverpool"
                }
            ],
            "date": 1601319600000,
            "externalId": "23203829",
            "hasLiveBetting": false
        }
        """

        payload = {'lang': "fi"}

        response = self._access_endpoint(
            EndPoint.single_draw_info_endpoint(draw_id), payload, method="GET")

        if not response:
            return None

        data = response.json()

        event = EventInfo()
        event.league = data.get('tournamentName')
        event.external_id = data.get('externalId')

        return event

    def place_bet(self, game: Game, bet: BetDecision, test=True) -> bool:
        """Place a bet, bet amount in cents

        Arguments:
            game: which draw to place the bet for
            bet: what to bet
            test: (optional) whether to use the API test endpoint
                    which does not actually place the bet, just checks
                    that it could have been placed
        """
        endpoint = EndPoint.place_wager_endpoint()
        if test:
            endpoint = EndPoint.place_wager_test_endpoint()

        payload = self.ebet_payload([game], [bet])

        response = self._access_endpoint(endpoint, payload=payload, method="POST")

        if not response:
            return False

        return True

    @staticmethod
    def ebet_payload(games: List[Game], bets: List[BetDecision]):
        """
        Payload for ebet wager:
        https://github.com/VeikkausOy/sport-games-robot/blob/master/doc/ebet-single-wager-request.json

        API payload:
            [
                {
                    "gameName": "EBET",
                    "requestId": "request-19",
                    "selections": [
                        {
                            "betType": "SINGLE",
                            "competitors": {
                                "main": [
                                    "1"
                                ],
                                "spare": [
                                    "310"
                                ]
                            },
                            "rowId": "150410",
                            "stake": 100,
                            "systemBetType": "SYSTEM"
                        },
                        {
                            "betType": "SINGLE",
                            "competitors": {
                                "main": [
                                    "3"
                                ],
                                "spare": [
                                    "220"
                                ]
                            },
                            "rowId": "150411",
                            "stake": 100,
                            "systemBetType": "SYSTEM"
                        }
                    ],
                    "type": "NORMAL"
                }
            ]
        """
        assert len(games) == len(bets), "Number of games has to match number of bets"

        def selected_play(game: Game, target: BetTarget):
            if target == BetTarget.HOME:
                return "1", game.home_odds
            if target == BetTarget.X:
                return "2", game.draw_odds
            if target == BetTarget.AWAY:
                return "3", game.away_odds
            raise TypeError("invalid bet target {}".format(target.value))

        data = []
        for game, bet in zip(games, bets):
            game_data = {
                "type": "NORMAL",
                "gameName": GameTypes.EBET.value,
            }
            # Loading of the data could be refactored into a
            # cleaner typed dataclass, for now ignore types
            game_data["selections"] = [{    # type: ignore
                "systemBetType": "NORMAL",
                "stake": bet.amount,
                "competitors": {
                    "main": [selected_play(game, bet.target)[0]],
                    "spare": [int(selected_play(game, bet.target)[1])],
                },
                "rowId": game.row_id,
            }]
            data.append(game_data)

        return data


def print_sport_info(client: VeikkausClient):
    """Print example info available for football"""
    # available sports
    sport_ids = client.sport_types()
    print(json.dumps(sport_ids, indent=4))
    # available categories for football
    fb_categories = client.sport_categories(1)
    print(json.dumps(fb_categories, indent=4))

    fb_tournaments = client.sport_tournaments(1, 2)
    print(json.dumps(fb_tournaments, indent=4))

    fb_tournament = client.sport_tournament_info(1, 2, 1)
    print(json.dumps(fb_tournament, indent=4))


def main():
    """A test function"""
    client = VeikkausClient()

    # get balances
    client.get_balance()

    # get upcoming EBET (Pitkäveto) draws
    games = client.upcoming_events(GameTypes.EBET)
    logger.info(games[0])
    logger.info("and %s other games", len(games) - 2)
    logger.info(games[-1])

    if not games:
        return

    # place bet on the first game
    game = games[-1]
    print("\n\nplacing bet for game:\n{}\n".format(game))

    success = client.place_bet(game, BetDecision(BetTarget.HOME, 100), test=True)

    if success:
        logger.info("\033[32mSUCCESS\033[0m bet placed")

    # print_sport_info(client)


if __name__ == "__main__":
    main()



"""A small script to invoke a client through all the supported
endpoints and save the sent out requests and responses.
"""
import json
from pathlib import Path
from typing import Any, Dict

import requests

from veikkaaja.endpoints import EndPoint
from veikkaaja.veikkaus_client import (BetDecision, BetTarget, GameTypes,
                                       VeikkausClient)


class SavingClient(VeikkausClient):
    """A small client that saves the outgoing and incoming
    API requests/responses under
    'api_requests/endpoint' and 'api_responsese/endpoint'
    """

    def save_outgoing_request(self, endpoint: EndPoint, payload: Dict[Any, Any]):
        """For testing, add and interface for saving the outgoing messages."""
        out_folder = Path(__file__).parent / "api_requests" / (
            endpoint.endpoint.replace('/', '.') + '.json')
        with out_folder.open('w') as file_handle:
            json.dump(payload, file_handle)

    def save_incoming_response(self, endpoint: EndPoint, response: requests.Response):
        """For testing, add and interface for saving the incoming responses."""
        out_folder = Path(__file__).parent / "api_responses" / (
            endpoint.endpoint.replace('/', '.') + ".json")
        with out_folder.open('w') as file_handle:
            json.dump(response.json(), file_handle)


def main():
    """A test function"""
    client = SavingClient()

    # get balances
    client.get_balance()

    # get upcoming EBET (Pitkäveto) draws
    games = client.upcoming_events(GameTypes.EBET)

    if not games:
        return

    # place bet on the first game
    game = games[-1]
    print("\n\nplacing bet for game:\n{}\n".format(game))

    success = client.place_bet(game, BetDecision(BetTarget.HOME, 100), test=True)
    assert success, "Could not place bet"

    # available sports
    client.sport_types()

    # available categories for football
    client.sport_categories(1)

    # tournaments for football and england
    client.sport_tournaments(1, 2)

    # info on the england main football league
    client.sport_tournament_info(1, 2, 1)


if __name__ == "__main__":
    main()


"""
veikkaaja module, here we set up the logging

export VEIKKAAJA_DEBUG=1 environment variable to
set the log level to logging.DEBUG.
"""
import logging
import logging.handlers
import os
import sys
from datetime import datetime

__version__ = "0.1.0"

LOGGING_INITIALIZED = False

if not LOGGING_INITIALIZED:
    logger = logging.getLogger('veikkaaja')

    logger.setLevel(logging.INFO)
    if os.environ.get('VEIKKAAJA_DEBUG', False):
        logger.setLevel(logging.DEBUG)

    class NiceFormatter(logging.Formatter):
        """Format: [  INFO ] 2020-10-17 10:42:41 | The message."""

        def format(self, record):
            record.current_time = datetime.now().strftime("%Y-%m-%d %H:%M:%S")
            return super().format(record)

    sys_out_handler = logging.StreamHandler(sys.stdout)
    sys_out_handler.setFormatter(
        NiceFormatter(fmt="[%(levelname)7s ] %(current_time)s | %(message)s"))

    logger.addHandler(sys_out_handler)

    LOGGING_INITIALIZED = True



"""Test whether the CI can access some basic API endpoints"""
import os
import unittest

from veikkaaja.veikkaus_client import VeikkausClient


class TestAccess(unittest.TestCase):
    """test access to API."""

    @unittest.skipIf('CI' in os.environ, "This is not currently run in Github CI.")
    def test_login(self):
        """Login is attempted upon client initialization"""

        client = VeikkausClient()
        self.assertIsNotNone(client.session)



"""A client for testing.

This mock client does not actually send any requests
over the network, but rather uses responses stored from
previous valid queries to the API
"""
import json
from pathlib import Path
from typing import Any, Dict, Union

import requests
import responses

from veikkaaja.endpoints import EndPoint
from veikkaaja.veikkaus_client import VeikkausClient


class MockClient(VeikkausClient):
    """Do not try to actually login"""

    def __init__(self):  # pylint: disable=super-init-not-called
        """Replace the original initialization
        that requires account information is given as
        arguments os as environment variables

        Do not try to login to the API

        Register all saved api responses as 'responses' as
        available endpoints.
        """

        saved_responses = (Path(__file__).parent / 'api_responses').glob('*.json')

        # For each saved actual json response from the real API
        # register a callback with responses that would return
        # the same json content that the real query would have returned.
        for response in saved_responses:
            # TODO: We just assume that we do not POST
            # and GET same endpoints.
            responses.add(
                responses.POST,
                EndPoint.API_ENDPOINT + "/" + response.stem.replace('.', '/'),
                match_querystring=False,
                json=json.loads(response.read_text()))
            responses.add(
                responses.GET,
                EndPoint.API_ENDPOINT + "/" + response.stem.replace('.', '/'),
                match_querystring=False,
                json=json.loads(response.read_text()))

    @responses.activate
    def _access_endpoint(self,
                         endpoint: EndPoint,
                         payload: Dict[str, Any] = None,
                         method="GET"):
        """
        Override the common entrypoint that sends out requests

        Check if we have stored a correct response from the API for this
        request and return that instead of trying to actually query
        a response from the API.
        """

        # check if we have the corresponding request/response files available
        request_file = Path(__file__).parent / 'api_requests' / endpoint.endpoint.replace(
            '/', '.')
        if request_file.exists():
            print("Found target request for {}".format(endpoint.endpoint))
            print(request_file.read_text())
        # TODO: Found a saved request, compare this request to it

        # we have registered the response with 'responses'
        # Now we just go and get it
        if method == "GET":
            response = requests.get(
                endpoint.url, headers=self.API_HEADERS, params=payload)
        elif method == "POST":
            response = requests.post(endpoint.url, headers=self.API_HEADERS, json=payload)
        else:
            raise RuntimeError("Unsupported method {}".format(method))

        if response.status_code != 200:
            return None
        return response



"""Keep the endpoints separate"""


class EndPoint:
    """Container for the API endpoints"""

    API_ENDPOINT = "https://www.veikkaus.fi/api"

    def __init__(self, endpoint_suffix: str):
        """
        Arguments:
            endpoint_suffix: the part of the endpoint url
                that comes after the API_ENDPOINT
        """
        self.endpoint = endpoint_suffix
        self.url = f"{self.API_ENDPOINT}/{self.endpoint}"

    def __repr__(self):
        """Only show the endpoint"""
        return self.url

    @classmethod
    def login_endpoint(cls):
        """place for initializing session v1/sessions"""
        return cls("bff/v1/sessions")

    @classmethod
    def account_info_endpoint(cls):
        """query account information v1/players/self/account"""
        return cls("v1/players/self/account")

    @classmethod
    def games_info_endpoint(cls):
        """get info of upcoming games"""
        return cls("odj/v2/sport-games/draws")

    @classmethod
    def single_event_info_endpoint(cls, event_id: int):
        """get info of upcoming games"""
        return cls(f"v1/sports/events/{event_id}")

    @classmethod
    def single_draw_info_endpoint(cls, draw_id: int):
        """get info of upcoming games"""
        return cls(f"odj/v2/sport-games/draws/{draw_id}")

    @classmethod
    def place_wager_test_endpoint(cls):
        """check if the placed bet is valid"""
        return cls("v1/sport-games/wagers/check")

    @classmethod
    def place_wager_endpoint(cls):
        """check if the placed bet is valid"""
        return cls("v1/sport-games/wagers")

    @classmethod
    def sport_type_code_endpoint(cls):
        """get available sport codes"""
        return cls("v1/sports")

    @classmethod
    def sport_categories_endpoint(cls, sport_id: int):
        """get available categories for a sport"""
        return cls(f"v1/sports/{sport_id}")

    @classmethod
    def sport_tournaments_endpoint(cls, sport_id: int, sport_category_id: int):
        """get available tournaments for sport and category"""
        return cls(f"v1/sports/{sport_id}/categories/{sport_category_id}")

    @classmethod
    def sport_tournament_info_endpoint(cls, sport_id: int,
                                       sport_category_id: int,
                                       tournament_id: int):
        """get info for a specific sport, category, and tournament."""
        return cls(
            f"v1/sports/{sport_id}/categories/{sport_category_id}/tournaments/{tournament_id}"
        )



"""Setting up veikkaaja."""
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="veikkaaja",
    version="0.1.0",
    author="Miika Mäkelä",
    author_email="makelanmiika@gmail.com",
    description="A wrapper for Veikkaus (veikkaus.fi) betting API.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/miikama/veikkaaja",
    packages=setuptools.find_packages(exclude='test'),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        'requests',
    ],
    extras_require= {
        'dev': [
            'pytest',
            'pylint',
            'yapf',
            'mypy',
            'responses',
        ]
    },
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
        ],
    },
)
